@@ -38,15 +38,13 @@ class DeepSpeedAutotuningConfig(DeepSpeedConfigObject):
                                      AUTOTUNING_FAST,
                                      AUTOTUNING_FAST_DEFAULT)
 
-        self.results_dir = os.path.abspath(
-            get_scalar_param(autotuning_dict,
-                             AUTOTUNING_RESULTS_DIR,
-                             AUTOTUNING_RESULTS_DIR_DEFAULT))
+        self.results_dir = get_scalar_param(autotuning_dict,
+                                            AUTOTUNING_RESULTS_DIR,
+                                            AUTOTUNING_RESULTS_DIR_DEFAULT)
         assert self.results_dir, "results_dir cannot be empty"
-        self.exps_dir = os.path.abspath(
-            get_scalar_param(autotuning_dict,
-                             AUTOTUNING_EXPS_DIR,
-                             AUTOTUNING_EXPS_DIR_DEFAULT))
+        self.exps_dir = get_scalar_param(autotuning_dict,
+                                         AUTOTUNING_EXPS_DIR,
+                                         AUTOTUNING_EXPS_DIR_DEFAULT)
         assert self.exps_dir, "exps_dir cannot be empty"
         self.overwrite = get_scalar_param(autotuning_dict,
                                           AUTOTUNING_OVERWRITE,
