@@ -15,13 +15,14 @@ from deepspeed.utils.timer import SynchronizedWallClockTimer
 
 from ..runtime.state_dict_factory import SDLoaderFactory
 from ..runtime.weight_quantizer import WeightQuantization
-from ..module_inject.replace_module import replace_transformer_layer, generic_injection
+from ..module_inject import replace_transformer_layer, generic_injection
 from ..comm.comm import init_distributed
 from ..pipe import PipelineModule
 from ..moe.utils import has_moe_layers
-from ..runtime.zero import GatheredParameters
 from ..module_inject import LinearAllreduce, LinearLayer, Normalize, ReplaceWithTensorSlicing
-from ..module_inject.replace_policy import DSPolicy
+from deepspeed.accelerator import get_accelerator
+from ..module_inject.policy import TransformerPolicy
+from ..module_inject.auto_tp import AutoTP
 
 DS_INFERENCE_ENABLED = False
 from torch import nn
@@ -50,8 +51,12 @@ class InferenceEngine(Module):
 
         self._get_model_config_generate(config)  # keep for weird backward compatibility
 
+        # patch model generate with ours if model uses it
+        if hasattr(self.module, "generate"):
+            self.generate = self._generate
+
         if hasattr(self.module, "config"):
-            DSPolicy.hf_model_config = self.module.config
+            TransformerPolicy.hf_model_config = self.module.config
 
         # todo: keep this self.injection_dict because we don't use to change config.injection_policy API
         # todo: this will get changed when Molly's PR on auto injection dict is merged
@@ -81,7 +86,7 @@ class InferenceEngine(Module):
         # This is a hack to remove the prepare_mask function on HF side for BLOOM architecture
         self.remove_mask_prepare_for_bloom()
 
-        if config.enable_cuda_graph:
+        if get_accelerator().device_name() == 'cuda' and config.enable_cuda_graph:
             assert pkg_version.parse(torch.__version__) >= pkg_version.parse("1.10"), \
                 "If you want to use cuda graph, please upgrade torch to at least v1.10"
 
@@ -112,7 +117,10 @@ class InferenceEngine(Module):
         if not config.replace_with_kernel_inject:
             config.checkpoint = None
 
+        # We only support three modes: 1) user specified policy for tensor-parallelism, 2) kernel injection (replace_with_kernel_inject), and 3) automatic tensor parallelism.
         if self.injection_dict:
+            # 1. User specified Tensor Parallelism
+            assert not config.replace_with_kernel_inject, "Cannot use both user specified injection policy and kernel injection"
             for client_module, injection_policy in self.injection_dict.items():
                 # construct the tuple and pass that instead of a string or dict.
                 if isinstance(injection_policy, str):
@@ -120,16 +128,29 @@ class InferenceEngine(Module):
                 else:
                     config.injection_policy_tuple = injection_policy
                 self._apply_injection_policy(config, client_module)
-        elif config.replace_method == 'auto':
-            self._apply_injection_policy(config)
+        else:
+            if config.replace_with_kernel_inject:
+                # 2. DeepSpeed Kernel Injection
+                self._apply_injection_policy(config)
+            else:
+                # 3. Automatic Tensor Parallelism
+                parser_dict = AutoTP.tp_parser(model)
+                print("AutoTP: ", parser_dict)
+                for client_module, injection_policy in parser_dict:
+                    if isinstance(injection_policy, str):
+                        config.injection_policy_tuple = (injection_policy, )
+                    else:
+                        config.injection_policy_tuple = injection_policy
+                    self._apply_injection_policy(config, client_module)
 
-        device = torch.cuda.current_device()
+        device = get_accelerator().current_device_name()
         self.module.to(device)
 
         if config.tensor_parallel.tp_size > 1:
-            _rng_state = torch.cuda.get_rng_state().to(torch.cuda.current_device())
+            _rng_state = get_accelerator().get_rng_state().to(
+                get_accelerator().current_device_name())
             dist.broadcast(_rng_state, 0)
-            torch.cuda.set_rng_state(_rng_state.cpu())
+            get_accelerator().set_rng_state(_rng_state.cpu())
 
         if config.tensor_parallel.tp_size > 1:
             assert not config.enable_cuda_graph, "Cuda graph is not supported for model parallelism"
@@ -149,8 +170,6 @@ class InferenceEngine(Module):
         self.config = getattr(self.module,
                               'config',
                               None) if config.config is None else config.config
-        # todo: clarify with Reza if this gets used anywhere
-        self.generate = getattr(self.module, 'generate', None)
 
     def remove_mask_prepare_for_bloom(self):
         if hasattr(self.module, 'transformer'):
@@ -161,7 +180,7 @@ class InferenceEngine(Module):
         if self.use_cuda_events:
             self.timers(INFERENCE_MODEL_TIMER).start()
         else:
-            torch.cuda.synchronize()
+            get_accelerator().synchronize()
             self._start = time.time()
 
     def _post_forward_hook(self, module, input, output):
@@ -169,7 +188,7 @@ class InferenceEngine(Module):
             self.timers(INFERENCE_MODEL_TIMER).stop()
             elapsed_time = self.timers(INFERENCE_MODEL_TIMER).elapsed(reset=True)
         else:
-            torch.cuda.synchronize()
+            get_accelerator().synchronize()
             self._end = time.time()
             elapsed_time = self._end - self._start
         self._model_times.append(elapsed_time)
@@ -179,7 +198,7 @@ class InferenceEngine(Module):
         if InferenceEngine.inference_mp_group is None:
             init_distributed()
             local_rank = int(os.getenv('LOCAL_RANK', '0'))
-            torch.cuda.set_device(local_rank)
+            get_accelerator().set_device(local_rank)
 
             ranks = [i for i in range(config.tensor_parallel.tp_size)]
             self.mp_group = dist.new_group(ranks)
@@ -273,35 +292,25 @@ class InferenceEngine(Module):
 
         def load(module, state_dict, prefix):
             args = (state_dict, prefix, {}, True, [], [], error_msgs)
-            if len(list(module.parameters())) > 0 and list(
-                    module.parameters())[0].numel() == 0:
-                with GatheredParameters(list(module.parameters(recurse=False)),
-                                        modifier_rank=0):
-                    if dist.get_rank() == 0:
-                        module._load_from_state_dict(*args)
-            else:
-                if hasattr(module, 'weight'):
-                    if 'query_key_value' in prefix:
-                        module.weight = self.mp_replace.qkv_copy(
-                            module.weight.data,
-                            state_dict[prefix + 'weight'])
-                    else:
-                        module.weight = self.mp_replace.copy(
-                            module.weight.data,
-                            state_dict[prefix + 'weight'])
-                else:
-                    module.norm.weight = self.mp_replace.copy(
-                        module.norm.weight.data,
+            if hasattr(module, 'weight'):
+                if 'query_key_value' in prefix:
+                    module.weight = self.mp_replace.qkv_copy(
+                        module.weight.data,
                         state_dict[prefix + 'weight'])
-                if prefix + 'bias' in self.key_list:
-                    if hasattr(module, 'norm'):
-                        module.norm.bias = self.mp_replace.copy(
-                            module.norm.bias,
-                            state_dict[prefix + 'bias'])
-                    else:
-                        data = state_dict[prefix + 'bias']
-                        data = data.to(torch.cuda.current_device())
-                        module.bias = self.mp_replace.copy(module.bias, data)
+                else:
+                    module.weight = self.mp_replace.copy(module.weight.data,
+                                                         state_dict[prefix + 'weight'])
+            else:
+                module.norm.weight = self.mp_replace.copy(module.norm.weight.data,
+                                                          state_dict[prefix + 'weight'])
+            if prefix + 'bias' in self.key_list:
+                if hasattr(module, 'norm'):
+                    module.norm.bias = self.mp_replace.copy(module.norm.bias,
+                                                            state_dict[prefix + 'bias'])
+                else:
+                    data = state_dict[prefix + 'bias']
+                    data = data.to(get_accelerator().current_device_name())
+                    module.bias = self.mp_replace.copy(module.bias, data)
 
         layer_policies = {
             nn.Linear: load,
@@ -402,7 +411,8 @@ class InferenceEngine(Module):
             for i in range(1, len(sd_loader)):
                 if not dist.is_initialized() or dist.get_rank() == 0:
                     print(f"loading checkpoint ({i})")
-                self.sd = torch.load(sd_loader[i], map_location='cuda')
+                self.sd = torch.load(sd_loader[i],
+                                     map_location=get_accelerator().device_name())
                 self.key_list = list(self.sd.keys())
                 self.load_model_with_checkpoint(self.module)
         else:
@@ -463,12 +473,12 @@ class InferenceEngine(Module):
 
     def _create_cuda_graph(self, *inputs, **kwargs):
         # warmup to create the workspace and cublas handle
-        cuda_stream = torch.cuda.Stream()
-        cuda_stream.wait_stream(torch.cuda.current_stream())
-        with torch.cuda.stream(cuda_stream):
+        cuda_stream = get_accelerator().Stream()
+        cuda_stream.wait_stream(get_accelerator().current_stream())
+        with get_accelerator().stream(cuda_stream):
             for i in range(3):
                 ret = self.module(*inputs, **kwargs)
-        torch.cuda.current_stream().wait_stream(cuda_stream)
+        get_accelerator().current_stream().wait_stream(cuda_stream)
 
         # create cuda_graph and assign static_inputs and static_outputs
         self._cuda_graphs = torch.cuda.CUDAGraph()
@@ -510,11 +520,12 @@ class InferenceEngine(Module):
             **kwargs: variable length keyword arguments
         """
         start = None
-        if self.model_profile_enabled and self._config.enable_cuda_graph:
-            torch.cuda.synchronize()
+        if self.model_profile_enabled and get_accelerator().device_name(
+        ) == 'cuda' and self._config.enable_cuda_graph:
+            get_accelerator().synchronize()
             start = time.time()
 
-        if self._config.enable_cuda_graph:
+        if get_accelerator().device_name() == 'cuda' and self._config.enable_cuda_graph:
             if self.cuda_graph_created:
                 outputs = self._graph_replay(*inputs, **kwargs)
             else:
@@ -524,8 +535,27 @@ class InferenceEngine(Module):
             outputs = self.module(*inputs, **kwargs)
 
         if self.model_profile_enabled and self._config.enable_cuda_graph:
-            torch.cuda.synchronize()
+            get_accelerator().synchronize()
             duration = time.time() - start
             self._model_times.append(duration)
 
         return outputs
+
+    def _generate(self, *inputs, **kwargs):
+        # Reset KV-cache at the beginning of generate
+        if hasattr(self.module, 'reset_cache'):
+            self.module.reset_cache()
+        num_beams = 1
+        if "generation_config" in kwargs:
+            gen_config = kwargs["generation_config"]
+            num_beams = getattr(gen_config, "num_beams", 1)
+        if "num_beams" in kwargs:
+            num_beams = kwargs["num_beams"]
+
+        if num_beams > 1:
+            raise NotImplementedError(
+                "DeepSpeed does not support `num_beams` > 1, if this is important to you please "
+                "add your request to: https://github.com/microsoft/DeepSpeed/issues/2506"
+            )
+
+        return self.module.generate(*inputs, **kwargs)
