@@ -3,13 +3,14 @@ Copyright 2022 The Microsoft DeepSpeed Team
 '''
 
 import torch
-from deepspeed.ops import op_builder
 import torch.nn as nn
 from deepspeed import comm as dist
 from deepspeed.utils.logging import log_dist
 
 from deepspeed.ops.transformer.inference.ds_mlp import DeepSpeedMLP
-from deepspeed.ops.transformer.inference.ds_attention import DeepSpeedSelfAttention
+from deepspeed.ops.transformer.inference.ds_attention import DeepSpeedSelfAttention, BloomSelfAttention
+from deepspeed.accelerator import get_accelerator
+from deepspeed.ops.op_builder import InferenceBuilder
 
 inference_cuda_module = None
 
@@ -38,8 +39,7 @@ class DeepSpeedTransformerInference(nn.Module):
                  quantize_scales=None,
                  quantize_groups=1,
                  merge_count=1,
-                 mlp_extra_grouping=False,
-                 qkv_merging=False):
+                 mlp_extra_grouping=False):
         super(DeepSpeedTransformerInference, self).__init__()
 
         self.config = config
@@ -49,18 +49,24 @@ class DeepSpeedTransformerInference(nn.Module):
         data_type = torch.half if config.fp16 else torch.float
         global inference_cuda_module
         if inference_cuda_module is None:
-            builder = op_builder.InferenceBuilder()
+            builder = InferenceBuilder()
             inference_cuda_module = builder.load()
 
         if DeepSpeedTransformerInference.layer_id == 1:
             log_dist(f"DeepSpeed-Inference config: {self.config.__dict__}", [0])
 
-        self.attention = DeepSpeedSelfAttention(self.config,
+        if self.config.bigscience_bloom:
+            self.attention = BloomSelfAttention(self.config,
                                                 mp_group,
                                                 quantize_scales,
                                                 quantize_groups,
-                                                merge_count,
-                                                qkv_merging)
+                                                merge_count)
+        else:
+            self.attention = DeepSpeedSelfAttention(self.config,
+                                                    mp_group,
+                                                    quantize_scales,
+                                                    quantize_groups,
+                                                    merge_count)
         self.mlp = DeepSpeedMLP(self.config,
                                 mp_group,
                                 quantize_scales,
@@ -68,7 +74,8 @@ class DeepSpeedTransformerInference(nn.Module):
                                 merge_count,
                                 mlp_extra_grouping)
 
-        device = torch.cuda.current_device()  #if config.bigscience_bloom else 'cpu'
+        device = get_accelerator().current_device_name(
+        )  # if config.bigscience_bloom else 'cpu'
         self.norm_w = nn.Parameter(torch.empty(self.config.hidden_size,
                                                dtype=data_type,
                                                device=device),
@@ -81,17 +88,24 @@ class DeepSpeedTransformerInference(nn.Module):
         self.allocate_workspace = inference_cuda_module.allocate_workspace_fp32 if (not config.fp16) else \
                                 inference_cuda_module.allocate_workspace_fp16
 
+    @classmethod
+    def reset_cache(cls):
+        if inference_cuda_module is not None:
+            inference_cuda_module.reset_cache()
+
     def forward(
             self,
-            input,
+            input=None,
             input_mask=None,
             attention_mask=None,
+            attn_mask=None,
             head_mask=None,
             layer_past=None,
             get_key_value=False,
             get_present=False,
             encoder_output=None,
             enc_dec_attn_mask=None,
+            x=None,
             encoder_hidden_states=None,
             encoder_attention_mask=None,
             use_cache=False,
@@ -101,6 +115,13 @@ class DeepSpeedTransformerInference(nn.Module):
             # This needs to be redesigned later!
             layer_head_mask=None,
             past_key_value=None):
+
+        if x is not None:
+            input = x
+
+        input_mask = (input_mask if attn_mask is None else
+                      attn_mask) if attention_mask is None else attention_mask
+
         # Allocate memory only on first layer forward
         if self.config.layer_id == 0:
             self.allocate_workspace(self.config.hidden_size,
@@ -159,7 +180,9 @@ class DeepSpeedTransformerInference(nn.Module):
         if get_present:
             output = (output, presents)
 
-        if self.config.return_tuple:
+        if self.config.return_single_tuple:
+            return (output, )
+        elif self.config.return_tuple:
             return output if type(output) is tuple else (output, attn_mask)
         else:
             return output
