@@ -4,6 +4,7 @@ from torch import nn
 from torch.nn import functional as F
 
 from torch.nn.parameter import Parameter
+from deepspeed.accelerator import get_accelerator
 
 
 class LinearAllreduce(nn.Module):
@@ -32,12 +33,12 @@ class LinearLayer(nn.Module):
             self.weight = Parameter(
                 torch.empty(weight_shape,
                             dtype=dtype,
-                            device=torch.cuda.current_device()))
+                            device=get_accelerator().current_device_name()))
 
             self.bias = Parameter(
                 torch.empty(weight_shape[0],
                             dtype=dtype,
-                            device=torch.cuda.current_device())) \
+                            device=get_accelerator().current_device_name())) \
                 if bias is not None else None
 
     def forward(self, input):
@@ -50,7 +51,9 @@ class LinearLayer(nn.Module):
 class Normalize(nn.Module):
     def __init__(self, dim, dtype=torch.float, eps=1e-5):
         super(Normalize, self).__init__()
-        self.norm = nn.LayerNorm(dim, eps=eps).to(dtype).to(torch.cuda.current_device())
+        self.norm = nn.LayerNorm(dim,
+                                 eps=eps).to(dtype).to(
+                                     get_accelerator().current_device_name())
         self.weight = self.norm.weight
         self.bias = self.norm.bias
 
@@ -65,7 +68,7 @@ class EmbeddingLayer(nn.Module):
             torch.empty(weight_shape[0],
                         weight_shape[1],
                         dtype=dtype,
-                        device=torch.cuda.current_device()))
+                        device=get_accelerator().current_device_name()))
 
     def forward(self, input):
         return F.embedding(input, self.weight)
