@@ -5,12 +5,12 @@ import deepspeed.comm as dist
 
 
 class csvMonitor(Monitor):
-    def __init__(self, monitor_config):
-        super().__init__(monitor_config)
+    def __init__(self, csv_config):
+        super().__init__(csv_config)
         self.filenames = []
-        self.enabled = monitor_config.csv_monitor_config.enabled
-        self.output_path = monitor_config.csv_monitor_config.output_path
-        self.job_name = monitor_config.csv_monitor_config.job_name
+        self.enabled = csv_config.enabled
+        self.output_path = csv_config.output_path
+        self.job_name = csv_config.job_name
         self.log_dir = self.setup_log_dir()
 
     def setup_log_dir(self, base=os.path.join(os.path.expanduser("~"), "csv_monitor")):
