@@ -15,7 +15,9 @@ class QuantizerBuilder(CUDAOpBuilder):
     def sources(self):
         return [
             'csrc/quantization/pt_binding.cpp',
-            'csrc/quantization/quantizer.cu',
+            'csrc/quantization/fake_quantizer.cu',
+            'csrc/quantization/quantize.cu',
+            'csrc/quantization/dequantize.cu',
         ]
 
     def include_paths(self):
