@@ -3,6 +3,8 @@ Copyright 2020 The Microsoft DeepSpeed Team
 """
 from .builder import CUDAOpBuilder
 
+import sys
+
 
 class FusedAdamBuilder(CUDAOpBuilder):
     BUILD_VAR = "DS_BUILD_FUSED_ADAM"
@@ -18,14 +20,18 @@ class FusedAdamBuilder(CUDAOpBuilder):
         return ['csrc/adam/fused_adam_frontend.cpp', 'csrc/adam/multi_tensor_adam.cu']
 
     def include_paths(self):
-        return ['csrc/includes']
+        return ['csrc/includes', 'csrc/adam']
 
     def cxx_args(self):
         args = super().cxx_args()
         return args + self.version_dependent_macros()
 
     def nvcc_args(self):
-        return ['-lineinfo',
-                '-O3',
+        nvcc_flags = ['-O3'] + self.version_dependent_macros()
+        if not self.is_rocm_pytorch():
+            nvcc_flags.extend([
+                '-allow-unsupported-compiler' if sys.platform == "win32" else '',
+                '-lineinfo',
                 '--use_fast_math'
-                ] + self.version_dependent_macros() + self.compute_capability_args()
+            ] + self.compute_capability_args())
+        return nvcc_flags
